/*
  Name: tools.cpp
  Copyright: Polecatsoft 2005
  Author: Master Lenman
  Date: 17.06.05 14:41
  Description: Contains useful macro functions,
  that can be used in all other files.
*/

#include "StdAfx.h"
#include <string>
#include <fstream>
using namespace std;

//Returns the file size of the file filename
//Can only be used for files smaller than 4GB
unsigned int fileSize(string filename)
{
	ifstream file(filename.c_str());
	if(file.fail())
		return 0;
	file.seekg(0, ios::end);
	return file.tellg();
}

//Returns false if the file filename can not be accessed (this normally means,
//that it doesn't exists)
bool fileExists(string filename)
{
	ifstream testStream(filename.c_str());
	return !testStream.fail();
}

//Below are functions of the keyStatesClass class

//Global instance of the keyStatesClass
keyStatesClass keyStates;

//Default constructor
keyStatesClass::keyStatesClass()
{
	resetAll();
}

//The function used to access the key value id
int keyStatesClass::operator [](int id)
{
	return getKey(id);
}

//Returns the value of a key
int keyStatesClass::getKey(int id)
{
	if(id >= KEY_MAX || id < 0)
		return 0;
	if(key[id] && !pressedKeys[id])
	{
		pressedKeys[id] = true;
		return 1;
	}
	if(!key[id])
		pressedKeys[id] = false;
	return 0;
}

//Returns true if ANY key was pressed
bool keyStatesClass::anyKey()
{
	bool retVal = false;
	for(int i = 0; i < KEY_MAX; ++i)
	{
		if(getKey(i))
			retVal = true;
	}
	return retVal;
}

//Sets all array values to false
void keyStatesClass::resetAll()
{
	for(int i = 0; i < KEY_MAX; ++i)
		pressedKeys[i] = false;
}

//Assign a value to the key id
void keyStatesClass::setState(int id, int value)
{
	pressedKeys[id] = value;
}

//Below are functions of the sound class

//different states the active variable can take
enum activeStates
{
	INACTIVE = 0,
	SAMPLEACTIVE = 1,
	MIDIACTIVE = 2,
	MP3ACTIVE = 3,
};

//Default constructor
sound::sound()
{
	sample = NULL;
	midi = NULL;
	mp3 = NULL;
	mp3Data = NULL;
	active = INACTIVE;
	autoPoll = false;
	paused = false;
}

//Destructor, calls destroy()
sound::~sound()
{
	destroy();
}

//Loads the file filename and turns autopoll on for mp3s if doPoll is true
bool sound::load(string filename, bool doPoll)
{
	destroy();
	//Load wav file
	if(
		filename[filename.length() - 3] == 'w' &&
		filename[filename.length() - 2] == 'a' &&
		filename[filename.length() - 1] == 'v')
	{
		sample = load_sample(filename.c_str());
		if(!sample)
		    return false;

		active = SAMPLEACTIVE;
	}
	//Load mp3
	else if(
		filename[filename.length() - 3] == 'm' &&
		filename[filename.length() - 2] == 'p' &&
		filename[filename.length() - 1] == '3')
	{
		#if 0
		int length = file_size(filename.c_str());
		if(!length)
		    return false;
		mp3Data = new char[length];

		PACKFILE *mp3readfile = pack_fopen(filename.c_str(), "r");
		pack_fread(mp3Data, length, mp3readfile);
		pack_fclose(mp3readfile);

		mp3 = almp3_create_mp3(mp3Data, length);
		if(!mp3)
			return false;
		if(doPoll)
		{
			almp3_start_autopoll_mp3(mp3, 100);
			autoPoll = true;
		}
		else
		{
			autoPoll = false;
		}
		active = MP3ACTIVE;
		#endif
		return false;
	}
	//Load midi
	else if(
		(
			filename[filename.length() - 3] == 'm' &&
			filename[filename.length() - 2] == 'i' &&
			filename[filename.length() - 1] == 'd'
		)
		||
		(
			filename[filename.length() - 4] == 'm' &&
			filename[filename.length() - 3] == 'i' &&
			filename[filename.length() - 2] == 'd' &&
			filename[filename.length() - 1] == 'i'
		))
	{
		midi = load_midi(filename.c_str());
		if(!midi)
		    return false;

		active = MIDIACTIVE;
	}
	else
	    return false;
	return true;
}

//Plays the sound with the given parameters, returns false on error
void sound::play(int loop, int vol, int pan, int freq)
{
	if(active == MIDIACTIVE)
	{
		play_midi(midi, loop);
	}
	else if(active == SAMPLEACTIVE)
	{
		play_sample(sample, vol, pan, freq, loop);
	}
	else if(active == MP3ACTIVE)
	{
		//almp3_play_ex_mp3(mp3, 24576, vol, pan, freq, loop);
	}
}

//Stops to play a sound
void sound::stop()
{
	if(active == SAMPLEACTIVE)
	{
		stop_sample(sample);
	}
	else if(active == MIDIACTIVE)
	{
		stop_midi();
	}
	else if(active == MP3ACTIVE)
	{
		#if 0
		almp3_stop_mp3(mp3);
		almp3_rewind_mp3(mp3);
		#endif
	}
}

//Alters the playing method of a sound
void sound::adjust(int loop, int vol, int pan, int freq)
{
	if(active == SAMPLEACTIVE)
	{
		adjust_sample(sample, vol, pan, freq, loop);
	}
	else if(active == MP3ACTIVE)
	{
		#if 0
        almp3_adjust_mp3(mp3, vol, pan, freq, loop);
        #endif
	}
}

//Frees memory allocated and stops playing
void sound::destroy()
{
	if(sample)
	{
		destroy_sample(sample);
		sample = NULL;
	}
	if(midi)
	{
		destroy_midi(midi);
		midi = NULL;
	}
	if(mp3)
	{
		#if 0
		if(autoPoll)
			almp3_stop_autopoll_mp3(mp3);
		autoPoll = false;

		almp3_destroy_mp3(mp3);
		mp3 = NULL;
		delete [] mp3Data;
		mp3Data = NULL;
		#endif
	}
	active = INACTIVE;
}

//Is there a sound currently loaded?
int sound::isActive()
{
	return active;
}

//Poll function for mp3 files
void sound::poll()
{
	if(active == MP3ACTIVE && !autoPoll)
	{
		#if 0
		almp3_poll_mp3(mp3);
		#endif
	}
}

#if 0
//Constructor that will call load(filename, doPoll)
bool sound::sound(string filename, bool doPoll)
{
	sound();
	return load(filename, doPoll);
}
#endif
